% code associated with Donovan and Lythe, "T cell and reticular network co-dependence in HIV infection", Journal of Theoretical Biology 2016. PDE-based model. contact g.donovan@auckland.ac.nz, code from www.math.auckland.ac.nz/~gdon030/code.html


function [Tend,Pend,Fend,Lmean,Ttotv,Ptotv,Ftotv,Ivec,Lvec,T,F,P,dt]=solvemefunc(params)


if(params.plotflag==1)
close all
%clear all

figure(1);
figure(2);
%spreadfigures;
end



NL=125*3;
Lmax=125;
Lvec=linspace(params.Lthresh,Lmax,NL)';
dL=Lvec(2)-Lvec(1);

NI=2000;
Imax=100;
Ivec=linspace(-Imax,Imax,NI)';
dI=Ivec(2)-Ivec(1);

% initial "spatial" vectors
Lvecorig=Lvec;
Ivecorig=Ivec;

% initial populations

T=0*Ivec; % (almost) nothing on network to start
T(1)=10;
P=0*Ivec;
F=0*Lvec;
P(round((9+Imax)/dI))=1; % P0 cells at IL7=9
nf=trapz(Ivec,P);
P=P*params.P0/nf;
F(round((10-params.Lthresh)/dL))=1; % F0 FRCs at LTb=10;
nf=trapz(Lvec,F);
F=F*params.F0/nf; % F0 FRCs
Ftotorig=trapz(Lvec,F);


Tmax=10*365;
dt=1e-2;
Nt=round(Tmax/dt);
dt=Tmax/Nt;

breaks=[];
kk=1;

Ptotv=zeros(1,Nt);
Ftotv=zeros(1,Nt);
Ttotv=zeros(1,Nt);

HIVcorrection=0;



for ii=1:Nt %% main loop over time


	if( ii*dt>(8*365) & ii*dt<(9*365) ) %HIV on test
		params.mu=params.muHIV;
		params.muF=params.muF_HIV;
		%%
		
	else
		params.mu=1/365;
		params.muF=0;
		%%
	end


	Ttot=trapz(Ivec,T);
	Ftot=trapz(Lvec,F);
	Ptot=trapz(Ivec,P);

	Ttotv(ii)=Ttot;
	Ftotv(ii)=Ftot;
	Ptotv(ii)=Ptot;

	Ishift= dt*params.nu*Ftot*gfunc(Ttot,params)/Ttot;  % follow characteristics for IL7 eqs
	Ivec=Ivec+Ishift;


	% finite diffs for diffusion version
	Fleft=circshift(F,1);
	Fleft(1)=0;
	Fright=circshift(F,-1);
	Fright(end)=0;
	Frightright=circshift(Fright,-1);
	Frightright(end)=0;
	Dcoeff=params.Dco*(params.beta+params.gamma*Ttot/Ftotorig);
	Fmean=trapz(Lvec,Lvec.*F)/Ftot;

	Fonestore=F(1);
	if ((params.beta-params.gamma*(Ttot/Ftotorig) + (Fmean-75)*(Fmean>75))>0) % moving left
		F=F+dt*( (params.beta-params.gamma*(Ttot/Ftotorig) + (Fmean-75)*(Fmean>75)).*(Fright-F)/(dL) + (Dcoeff/2)*(Fright-2*F+Fleft)/(dL^2) - params.muF*F);
	else % moving right
		F=F+dt*( (params.beta-params.gamma*(Ttot/Ftotorig) + (Fmean-75)*(Fmean>75)).*(F-Fleft)/(dL) + (Dcoeff/2)*(Fright-2*F+Fleft)/(dL^2) - params.muF*F);
	end

	F(1)=0; % BC on the left

	T=T+dt*(-(params.mu+params.co)*T + params.rho*P);

	P2I=interp1(Ivec,P,2*Ivec,'linear',0);

	P=P+dt*(-params.mu*P + params.co*T - params.rho*P + 4*params.lambda*P2I.*myheaviside(Ivec-params.Ihat/2) - myheaviside(Ivec-params.Ihat).*params.lambda.*P);
	if (ii*dt<=365)
		plotstep=1000;
	else
		plotstep=10000;
	end
	if(mod(ii,plotstep)==0)
	if(params.plotflag==1)
	plotme(Ivec,Lvec,F,T,P,ii*dt);
	if(kk<=length(breaks))
	if(ii*dt>breaks(kk))
		kk=kk+1;
		ii*dt
		pause;
	end
	end
	end
	end
end

	if(params.plotflag==1)
	plotme(Ivec,Lvec,F,T,P,ii*dt);
	end

Tend=trapz(Ivec,T);
Pend=trapz(Ivec,P);
Fend=trapz(Lvec,F);
Lmean=trapz(Lvec,F.*Lvec)/Fend;











